<?php

namespace Drupal\media_alias_display\Controller;

use Drupal\Core\Entity\Controller\EntityViewController;
use Drupal\Core\Entity\EntityInterface;
use Drupal\file\Entity\File;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Drupal\Core\Url;
use Drupal\media\Entity\Media;

/**
 * DisplayController class.
 */
class DisplayController extends EntityViewController {

  /**
   *
   */
  public function view(EntityInterface $media, $view_mode = 'full', $langcode = NULL) {
    $current_path = \Drupal::service('path.current')->getPath();
    $path = \Drupal::request()->getRequestUri();

    $alias = \Drupal::service('path_alias.manager')
      ->getPathByAlias($current_path);
    $params = Url::fromUri("internal:" . $alias)->getRouteParameters();
    $entity_type = key($params);
    $mediaID = $params[$entity_type];
    $media = Media::load($mediaID);

    if ($media == NULL) {
      \Drupal::logger('media_alias_display')
        ->notice("Can't find media object for " . $current_path);
      throw new NotFoundHttpException("Can't find media object.");
    }

    if (strpos($path, "edit-media") !== FALSE && !\Drupal::currentUser()
      ->isAnonymous()) {
      return new RedirectResponse('/media/' . $mediaID . '/edit');
    }

    $source = $media->getSource();
    $config = $source->getConfiguration();
    $field = $config['source_field'];

    $fid = $media->{$field}->target_id;

    // If media has no file item.
    if (!$fid) {
      \Drupal::logger('media_alias_display')
        ->notice("The media item requested has no file referenced/uploaded for " . $current_path);
      return parent::view($media, $view_mode);
    }

    $file = File::load($fid);

    // Or file entity could not be loaded.
    if (!$file) {
      \Drupal::logger('media_alias_display')
        ->notice("File id could not be loaded for " . $current_path);
      return parent::view($media, $view_mode);
    }

    $uri = $file->getFileUri();
    $filename = $file->getFilename();

    // Or item does not exist on disk.
    if (!file_exists($uri)) {
      \Drupal::logger('media_alias_display')
        ->notice("File does not exist for " . $current_path);
      return parent::view($media, $view_mode);
    }

    $response = new BinaryFileResponse($uri);
    $response->setContentDisposition(
      ResponseHeaderBag::DISPOSITION_INLINE,
      $filename
    );

    return $response;
  }

}
